#
# pushare - P2P file sharing system
#
# Copyright (c) 2004 Satoshi Fukutomi <fuktommy@inter7.jp>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. Neither the name of the University nor the names of its contributors
#    may be used to endorse or promote products derived from this software
#    without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# $Id: Interface.pm,v 1.2 2004/12/17 15:56:48 fuktommy Exp $
#

package PuShare::Interface;
use strict;

require Exporter;
use vars qw(@ISA @EXPORT);
@ISA = qw(Exporter);
@EXPORT = qw(	escape encode decode printHTMLHeader print302
		printError print404 args dprint printHeader
);

sub escape($) {
	local($_) = @_;
	s/&/&amp;/g;
	s/</&lt;/g;
	s/>/&gt;/g;
	s/\r//g;
	s/\n/<br \/>/g;
	s/"/&quot;/g;
	return $_;
}

sub dprint(@) {
	print STDOUT @_, "\n";
}

sub printHeader($) {
	my ($stat) = @_;
	local $, = "\r\n";
	alarm $PuShare::Config::timeout;
	print "HTTP/1.0 $stat",
	      "Server: pushare/$PuShare::Config::version",
	      "Connection: Close",
	      "";
	alarm 0;
}

sub printHTMLHeader($) {
	my($title) = @_;
	printHeader("200 OK");
	print "Content-Type: text/html; charset=UTF-8\r\n",
	      "Content-Language: ja\r\n",
	      "\r\n";
	print '<?xml version="1.0" encoding="UTF-8"?>',
	      '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN"',
	      ' "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">',
	      '<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="ja">',
	      '<head>',
	      "<title>$title</title>",
	      "<link rev='made' href='mailto:fuktommy\@inter7.jp' />",
	      "<link rel='contents' href='/gw/' />",
	      "<style type='text/css'>",
	      " ul { list-style: none; }",
	      " h1 { color: red; }",
	      " em { font-size: x-large }",
	      " p  { margin: 0px; padding: 0px }",
	      " .name { color: green; font-weight: bold; }",
	      "</style></head>";
	print "<body><p>",
	      "<a href='/gw/cache'>CACHE</a> | ",
	      "<a href='/gw/search'>SEARCH</a>",
	      "</p>";

}

sub print302($) {
	my($next) = @_;
	printHeader("200 OK");
	print "Content-Type: text/html; charset=UTF-8\r\n",
	      "Content-Language: ja\r\n",
	      "\r\n";
	print '<?xml version="1.0" encoding="UTF-8"?>',
	      '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN"',
	      ' "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">',
	      '<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="ja">',
	      '<head>',
	      '<meta http-equiv="Content-Language" content="ja" />',
	      '<meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />',
	      '<meta http-equiv="Content-Script-Type" content="text/javascript" />',
	      "<title>moved</title>",
	      "<link rev='made' href='mailto:fuktommy\@inter7.jp' />",
	      "<link rel='contents' href='/gw/' />",
	      "</head>";
	print "<body><p>Click and jump to <a href='$next'>$next</a></p>\n",
	      "<script type='text/javascript'>window.location.href = '$next';</script>",
	      "</body></html>\n";
}

sub encode($) {			# from YukiWiki
	my ($s) = @_;
	$s =~ s/[^-_\w.]/'%' . uc(unpack('H2', $&))/eg;
	return $s;
}

sub decode($) {			# from YukiWiki
	my ($s) = @_;
	$s =~ tr/+/ /;
	$s =~ s/%([A-Fa-f0-9][A-Fa-f0-9])/pack("C", hex($1))/eg;
	return $s;
}

sub printError($) {
	my($stat) = @_;
	printHeader($stat);
	print "Content-Type: text/plain\r\n",
	      "\r\n",
	      "$stat\n";
}

sub print404() {
	printHeader("404 Not Found");
	print "Content-Type: text/plain\r\n",
	      "Content-Length: 0\r\n",
	      "\r\n";
}

sub args($) {
	local $_ = $_[0];
	local @_ = split /&/;
	my %arg = ();
	foreach (@_) {
		local @_ = split /=/;
		if (defined $_[1]) {
			$_[1] =~ s/\+/ /g;
			$_[1] =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
			$arg{$_[0]} = $_[1];
		} else {
			$arg{$_[0]} = "";
		}
	}
	return %arg;
}


1;
